@@ -1,5 +1,6 @@
 pirate = {
     "build plate": "deck",
+    "buildplate": "deck",
     "quit": "abandon ship",
     "back": "avast",
     "nozzle": "muzzle",
@@ -43,9 +44,24 @@ pirate = {
     "printing": "scribblin'",
     "load": "haul",
     "connect to": "board",
+    "connecting": "boarding",
     "collects": "hoards",
     "prime tower": "buoy",
     "change log": "captain's log",
     "my": "me",
-    "removable drive": "life boat"
+    "removable drive": "life boat",
+    "print core": "scribbler",
+    "printcore": "scribbler",
+    "abort": ["maroon", "abandon"],
+    "aborting": ["marooning", "abandoning"],
+    "aborted": ["marooned", "abandoned"],
+    "connected": ["anchored", "moored"],
+    "developer": "scurvy dog",
+    "wizard": "cap'n",
+    "active leveling": "keelhauling",
+    "download": "plunder",
+    "downloaded": "plundered",
+    "caution hot surface": "fire in the hole!",
+    "type": "sort",
+    "spool": "barrel",
 }
