@@ -1,4 +1,4 @@
-# Copyright (c) 2016 Ultimaker B.V.
+# Copyright (c) 2017 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import pytest
@@ -24,8 +24,8 @@ def definition_container():
     return result
 
 test_deserialize_data = [
-    ("basic.def.json", { "name": "Test", "metadata": {}, "settings": {} }),
-    ("metadata.def.json", { "name": "Test", "metadata": { "author": "Ultimaker", "category": "Test" }, "settings": {} }),
+    ("basic_definition.def.json", { "name": "Test", "metadata": {}, "settings": {} }),
+    ("metadata_definition.def.json", { "name": "Test", "metadata": { "author": "Ultimaker", "category": "Test" }, "settings": {} }),
     ("single_setting.def.json", { "name": "Test", "metadata": {}, "settings": { "test_setting": { "label": "Test", "default_value": 10, "description": "A Test Setting" } } }),
     ("multiple_settings.def.json", { "name": "Test", "metadata": {}, "settings": {
         "test_setting_0": { "label": "Test 0", "default_value": 10, "description": "A Test Setting" },
@@ -50,7 +50,6 @@ test_deserialize_data = [
 ]
 @pytest.mark.parametrize("file,expected", test_deserialize_data)
 def test_deserialize(file, expected, definition_container):
-    json = ""
     with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), "definitions", file)) as data:
         json = data.read()
 
@@ -263,11 +262,11 @@ def test_serialize(definition_container):
     _test_serialize_cycle(definition_container)
 
     # Change the name.
-    definition_container._name = "Bla!"
+    definition_container._metadata["name"] = "Bla!"
     _test_serialize_cycle(definition_container)
-    definition_container._name = "[\"\n{':" # Some characters that might need to be escaped.
+    definition_container._metadata["name"] = " [\"\n{':" # Some characters that might need to be escaped.
     _test_serialize_cycle(definition_container)
-    definition_container._name = "ルベン" # From a different character set (UTF-8 test).
+    definition_container._metadata["name"] = "ルベン" # From a different character set (UTF-8 test).
     _test_serialize_cycle(definition_container)
 
     # Add some metadata.
@@ -299,7 +298,7 @@ def test_serialize(definition_container):
 #
 #   \param definition_container A new definition container from a fixture.
 def test_serialize_with_ignored_metadata_keys(definition_container):
-    ignored_metadata_keys = ["secret", "secret2"]
+    ignored_metadata_keys = {"secret", "secret2"}
     # Add some metadata.
     definition_container.getMetaData()["author"] = "Testy McTesticle"
     definition_container.getMetaData()["escape_test"] = "[\"\n{':"
@@ -365,27 +364,30 @@ def _createSettingDefinition(properties):
 #   serialises the container to a string and deserialises it again from that
 #   string. Then it verifies that all properties are still the same.
 #
-#   \param definition_container A defintion container to test the serialisation
+#   \param definition_container A definition container to test the serialisation
 #   of.
 #   \param ignored_metadata_keys A list of keys in metadata that will be
 #   ignored during serialization.
 def _test_serialize_cycle(definition_container, ignored_metadata_keys = None):
     # Don't verify the ID. It must be unique, so it must be different.
     name = definition_container.getName()
-    metadata = {key: value for key, value in definition_container.getMetaData().items()}
+    metadata = {key: value for key, value in definition_container.getMetaData().items() if key != "id"}
     definitions = definition_container.definitions
     # No need to verify the internationalisation catalogue.
 
     serialised = definition_container.serialize(ignored_metadata_keys = ignored_metadata_keys)
     deserialised = UM.Settings.DefinitionContainer.DefinitionContainer(str(uuid.uuid4()))
     deserialised.deserialize(serialised)
+    deserialised_metadata = {key: value for key, value in deserialised.getMetaData().items() if key != "id"}
 
-    # remove ignored keys from metadata dict
+    # remove ignored keys from metadata dicts
     if ignored_metadata_keys:
         for key in ignored_metadata_keys:
             if key in metadata:
                 del metadata[key]
+            if key in deserialised_metadata:
+                del deserialised_metadata[key]
 
     assert name == deserialised.getName()
-    assert metadata == deserialised.getMetaData()
+    assert metadata == deserialised_metadata
     assert definitions == deserialised.definitions
